@@ -26,6 +26,7 @@
 #include "ephy-embed-prefs.h"
 #include "ephy-embed-utils.h"
 #include "ephy-file-helpers.h"
+#include "ephy-langs.h"
 
 typedef struct
 {
@@ -133,6 +134,89 @@ webkit_pref_callback_user_stylesheet (GConfClient *client,
   g_free (uri);
 }
 
+static char *
+webkit_pref_get_internal_user_agent (void)
+{
+  char *user_agent;
+  char *webkit_user_agent;
+  char *vendor_user_agent;
+  GKeyFile *branding_keyfile;
+
+  vendor_user_agent = NULL;
+  branding_keyfile = g_key_file_new ();
+
+  if (g_key_file_load_from_file (branding_keyfile, SHARE_DIR"/branding.conf",
+                                 G_KEY_FILE_NONE, NULL)) {
+    char *vendor;
+    char *vendor_sub;
+    char *vendor_comment;
+
+    vendor = g_key_file_get_string (branding_keyfile,
+                                    "User Agent", "Vendor", NULL);
+    vendor_sub = g_key_file_get_string (branding_keyfile,
+                                        "User Agent", "VendorSub", NULL);
+    vendor_comment = g_key_file_get_string (branding_keyfile,
+                                            "User Agent", "VendorComment", NULL);
+
+    if (vendor) {
+      vendor_user_agent = g_strconcat (vendor,
+                                       vendor_sub ? "/" : "",
+                                       vendor_sub ? vendor_sub : "",
+                                       vendor_comment ? " (" : "",
+                                       vendor_comment ? vendor_comment : "",
+                                       vendor_comment ? ")" : "",
+                                       NULL);
+    }
+
+    g_free (vendor);
+    g_free (vendor_sub);
+    g_free (vendor_comment);
+  }
+
+  g_key_file_free (branding_keyfile);
+
+  g_object_get (settings, "user-agent", &webkit_user_agent, NULL);
+
+  user_agent = g_strconcat (webkit_user_agent, " ",
+                            vendor_user_agent ? vendor_user_agent : "",
+                            vendor_user_agent ? " " : "",
+                            "Epiphany/"VERSION,
+                            NULL);
+
+  g_free (vendor_user_agent);
+  g_free (webkit_user_agent);
+
+  return user_agent;
+}
+
+static void
+webkit_pref_callback_user_agent (GConfClient *client,
+                                 guint cnxn_id,
+                                 GConfEntry *entry,
+                                 gpointer data)
+{
+  GConfValue *gcvalue;
+  const char *value = NULL;
+  static char *internal_user_agent = NULL;
+  char *webkit_pref = data;
+
+  gcvalue = gconf_entry_get_value (entry);
+
+  /* happens on initial notify if the key doesn't exist */
+  if (gcvalue != NULL &&
+      gcvalue->type == GCONF_VALUE_STRING) {
+      value = gconf_value_get_string (gcvalue);
+  }
+
+  if (value == NULL || value[0] == '\0') {
+    if (internal_user_agent == NULL)
+      internal_user_agent = webkit_pref_get_internal_user_agent ();
+
+    g_object_set (settings, webkit_pref, internal_user_agent, NULL);
+  } else
+    g_object_set (settings, webkit_pref, value, NULL);
+}
+
 static void
 webkit_pref_callback_font_size (GConfClient *client,
                                 guint cnxn_id,
@@ -194,6 +278,57 @@ webkit_pref_callback_font_family (GConfClient *client,
   }
 }
 
+/* Based on Christian Persch's code from gecko backend of epiphany
+   (old transform_accept_languages_list() function) */
+static void
+webkit_pref_callback_accept_languages (GConfClient *client,
+                                       guint cnxn_id,
+                                       GConfEntry *entry,
+                                       gpointer data)
+{
+  SoupSession *session;
+  GConfValue *gcvalue;
+  GArray *array;
+  GSList *languages, *l;
+  char **langs;
+  char *langs_str;
+  char *webkit_pref;
+
+  webkit_pref = data;
+  gcvalue = gconf_entry_get_value (entry);
+  if (gcvalue == NULL ||
+      gcvalue->type != GCONF_VALUE_LIST ||
+      gconf_value_get_list_type (gcvalue) != GCONF_VALUE_STRING)
+    return;
+
+  languages = gconf_value_get_list (gcvalue);
+
+  array = g_array_new (TRUE, FALSE, sizeof (char *));
+
+  for (l = languages; l != NULL; l = l->next) {
+      const char *lang = gconf_value_get_string ((GConfValue *) l->data);
+
+      if ((lang != NULL) && !g_strcmp0 (lang, "system")) {
+          ephy_langs_append_languages (array);
+        } else if (lang != NULL && lang[0] != '\0') {
+          char *str = g_ascii_strdown (lang, -1);
+          g_array_append_val (array, str);
+        }
+    }
+
+  ephy_langs_sanitise (array);
+
+  langs = (char **) g_array_free (array, FALSE);
+  langs_str = g_strjoinv (", ", langs);
+
+  /* Update Soup session */
+  session = webkit_get_default_session ();
+  g_object_set (G_OBJECT (session), webkit_pref, langs_str, NULL);
+
+  g_strfreev (langs);
+  g_free (langs_str);
+}
+
 static const PrefData webkit_pref_entries[] =
   {
     { CONF_RENDERING_FONT_MIN_SIZE,
@@ -231,7 +366,13 @@ static const PrefData webkit_pref_entries[] =
       webkit_pref_callback_boolean },
     { CONF_SECURITY_ALLOW_POPUPS,
       "javascript-can-open-windows-automatically",
-      webkit_pref_callback_boolean }
+      webkit_pref_callback_boolean },
+    { CONF_RENDERING_LANGUAGE,
+      "accept-language",
+      webkit_pref_callback_accept_languages },
+    { CONF_USER_AGENT,
+      "user-agent",
+      webkit_pref_callback_user_agent }
   };
 
 static void
@@ -251,7 +392,12 @@ ephy_embed_prefs_init (void)
   settings = webkit_web_settings_new ();
 
   /* Hardcoded settings */
-  g_object_set (settings, "auto-shrink-images", FALSE, NULL);
+  g_object_set (settings,
+                "auto-shrink-images", FALSE,
+                "enable-default-context-menu", FALSE,
+                "enable-site-specific-quirks", TRUE,
+                "enable-page-cache", TRUE,
+                NULL);
 
   /* Connections */
   connections = g_malloc (sizeof (guint) * G_N_ELEMENTS (webkit_pref_entries));
