@@ -1,3 +1,4 @@
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
  *  Copyright © 2004 Christian Persch
  *  Copyright © 2005 Philip Langdale
@@ -16,7 +17,6 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- *  $Id: ephy-link-action.c 6952 2007-03-11 19:42:02Z chpe $
  */
 
 #include "config.h"
@@ -27,12 +27,11 @@
 #include "ephy-debug.h"
 #include "ephy-gui.h"
 
-#include <gtk/gtkbutton.h>
-#include <gtk/gtkmain.h>
-#include <gtk/gtkmenuitem.h>
-#include <gtk/gtkmenutoolbutton.h>
+#include <gtk/gtk.h>
 
-static GObjectClass *parent_class;
+G_DEFINE_TYPE_WITH_CODE (EphyLinkAction, ephy_link_action, GTK_TYPE_ACTION,
+                         G_IMPLEMENT_INTERFACE (EPHY_TYPE_LINK,
+                                                NULL))
 
 static gboolean
 proxy_button_press_event_cb (GtkButton *button,
@@ -52,7 +51,7 @@ proxy_button_release_event_cb (GtkButton *button,
 			       GdkEventButton *event,
 			       EphyLinkAction *action)
 {
-	/**
+	/*
 	 * We do not use ephy_gui_is_middle_click() here because
 	 * that also catches ctrl + left_click which already
 	 * triggers an activate event for all proxies.
@@ -70,7 +69,7 @@ get_event_widget (GtkWidget *proxy)
 {
 	GtkWidget *widget;
 
-	/**
+	/*
 	 * Finding the interesting widget requires internal knowledge of
 	 * the widgets in question. This can't be helped, but by keeping
 	 * the sneaky code in one place, it can easily be updated.
@@ -82,13 +81,17 @@ get_event_widget (GtkWidget *proxy)
 	}
 	else if (GTK_IS_MENU_TOOL_BUTTON (proxy))
 	{
-		/**
+		/*
 		 * The menu tool button's button is the first child
 		 * of the child hbox.
 		 */
-		GtkContainer *container =
-			GTK_CONTAINER (gtk_bin_get_child (GTK_BIN (proxy)));
-		widget = GTK_WIDGET (gtk_container_get_children (container)->data);
+		GtkContainer *container;
+		GList *children;
+	  
+		container = GTK_CONTAINER (gtk_bin_get_child (GTK_BIN (proxy)));
+		children = gtk_container_get_children (container);
+		widget = GTK_WIDGET (children->data);
+		g_list_free (children);
 	}
 	else if (GTK_IS_TOOL_BUTTON (proxy))
 	{
@@ -126,7 +129,7 @@ ephy_link_action_connect_proxy (GtkAction *action, GtkWidget *proxy)
 				  action);
 	}
 
-	GTK_ACTION_CLASS (parent_class)->connect_proxy (action, proxy);
+	GTK_ACTION_CLASS (ephy_link_action_parent_class)->connect_proxy (action, proxy);
 }
 
 static void
@@ -147,7 +150,13 @@ ephy_link_action_disconnect_proxy (GtkAction *action, GtkWidget *proxy)
 						      action);
 	}
 
-	GTK_ACTION_CLASS (parent_class)->disconnect_proxy (action, proxy);
+	GTK_ACTION_CLASS (ephy_link_action_parent_class)->disconnect_proxy (action, proxy);
+}
+
+static void
+ephy_link_action_init (EphyLinkAction *action)
+{
+        /* Empty, needed for G_DEFINE_TYPE macro */
 }
 
 static void
@@ -155,86 +164,26 @@ ephy_link_action_class_init (EphyLinkActionClass *class)
 {
 	GtkActionClass *action_class = GTK_ACTION_CLASS (class);
 
-	parent_class = g_type_class_peek_parent (class);
-
 	action_class->connect_proxy = ephy_link_action_connect_proxy;
 	action_class->disconnect_proxy = ephy_link_action_disconnect_proxy;
 }
 
-GType
-ephy_link_action_get_type (void)
+static void
+ephy_link_action_group_class_init (EphyLinkActionGroupClass *klass)
 {
-	static GType type = 0;
-
-	if (G_UNLIKELY (type == 0))
-	{
-		const GTypeInfo our_info =
-		{
-			sizeof (EphyLinkActionClass),
-			NULL, /* base_init */
-			NULL, /* base_finalize */
-			(GClassInitFunc) ephy_link_action_class_init,
-			NULL,
-			NULL, /* class_data */
-			sizeof (EphyLinkAction),
-			0,   /* n_preallocs */
-			NULL /* instance_init */
-		};
-		const GInterfaceInfo link_info = 
-		{
-			NULL,
-			NULL,
-			NULL
-		};
-
-		type = g_type_register_static (GTK_TYPE_ACTION,
-					       "EphyLinkAction",
-					       &our_info, G_TYPE_FLAG_ABSTRACT);
-		g_type_add_interface_static (type,
-					     EPHY_TYPE_LINK,
-					     &link_info);
-	}
-
-	return type;
+        /* Empty, needed for G_DEFINE_TYPE macro */
 }
 
-GType
-ephy_link_action_group_get_type (void)
+static void
+ephy_link_action_group_init (EphyLinkActionGroup *action_group)
 {
-	static GType type = 0;
-
-	if (G_UNLIKELY (type == 0))
-	{
-		const GTypeInfo our_info =
-		{
-			sizeof (EphyLinkActionGroupClass),
-			NULL, /* base_init */
-			NULL, /* base_finalize */
-			NULL, /* class_init */
-			NULL,
-			NULL, /* class_data */
-			sizeof (EphyLinkActionGroup),
-			0,   /* n_preallocs */
-			NULL /* instance_init */
-		};
-		const GInterfaceInfo link_info = 
-		{
-			NULL,
-			NULL,
-			NULL
-		};
-
-		type = g_type_register_static (GTK_TYPE_ACTION_GROUP,
-					       "EphyLinkActionGroup",
-					       &our_info, 0);
-		g_type_add_interface_static (type,
-					     EPHY_TYPE_LINK,
-					     &link_info);
-	}
-
-	return type;
+        /* Empty, needed for G_DEFINE_TYPE macro */
 }
 
+G_DEFINE_TYPE_WITH_CODE (EphyLinkActionGroup, ephy_link_action_group, GTK_TYPE_ACTION_GROUP,
+                         G_IMPLEMENT_INTERFACE (EPHY_TYPE_LINK,
+                                                NULL))
+
 EphyLinkActionGroup *
 ephy_link_action_group_new (const char * name)
 {
