@@ -88,7 +88,7 @@ new_history_menu_item (const char *origtext,
 
 	item = gtk_image_menu_item_new_with_label (origtext);
 
-	label = GTK_LABEL (GTK_BIN (item)->child);
+	label = GTK_LABEL (gtk_bin_get_child (GTK_BIN (item)));
 	gtk_label_set_ellipsize (label, PANGO_ELLIPSIZE_END);
 	gtk_label_set_max_width_chars (label, MAX_LABEL_LENGTH);
 
@@ -236,13 +236,20 @@ build_back_or_forward_menu (EphyNavigationAction *action)
 	{
 		GtkWidget *item;
 		WebKitWebHistoryItem *hitem;
-		const char *title, *url;
+		const char *url;
+		char *title;
 
 		hitem = (WebKitWebHistoryItem*)l->data;
 		url = webkit_web_history_item_get_uri (hitem);
-		title = webkit_web_history_item_get_title (hitem);
 
-		item = new_history_menu_item (title ? title : url, url);
+		title = g_strdup (webkit_web_history_item_get_title (hitem));
+
+		if ((title == NULL || g_strstrip (title)[0] == '\0'))
+			item = new_history_menu_item (url, url);
+		else
+			item = new_history_menu_item (title, url);
+
+		g_free (title);
 
 		g_object_set_data_full (G_OBJECT (item), HISTORY_ITEM_DATA_KEY,
 					g_object_ref (hitem), g_object_unref);
@@ -285,7 +292,7 @@ build_up_menu (EphyNavigationAction *action)
     	history = EPHY_HISTORY
 		(ephy_embed_shell_get_global_history (embed_shell));
 
-	list = ephy_web_view_get_go_up_list (EPHY_GET_EPHY_WEB_VIEW_FROM_EMBED (embed));
+	list = ephy_web_view_get_go_up_list (ephy_embed_get_web_view (embed));
 
 	for (l = list; l != NULL; l = l->next)
 	{
@@ -430,7 +437,7 @@ ephy_navigation_action_activate (GtkAction *gtk_action)
 	{
 		GSList *up_list;
 		
-		up_list = ephy_web_view_get_go_up_list (EPHY_GET_EPHY_WEB_VIEW_FROM_EMBED (embed));
+		up_list = ephy_web_view_get_go_up_list (ephy_embed_get_web_view (embed));
 		ephy_link_open (EPHY_LINK (action),
 				up_list->data,
 				NULL,
