@@ -241,7 +241,7 @@ prefs_dialog_show_help (EphyDialog *dialog)
 	id = gtk_notebook_get_current_page (GTK_NOTEBOOK (notebook));
 	id = CLAMP (id, 0, 3);
 
-	ephy_gui_help (GTK_WINDOW (window), "epiphany", help_preferences[id]);
+	ephy_gui_help (window, help_preferences[id]);
 }
 
 static void
@@ -616,7 +616,8 @@ setup_add_language_dialog (PrefsDialog *pd)
 				  add_lang_props[LANGUAGE_DIALOG].id, &window,
 				  NULL);
 
-	gtk_window_group_add_window (GTK_WINDOW (parent)->group, GTK_WINDOW (window));
+	gtk_window_group_add_window (gtk_window_get_group (GTK_WINDOW (parent)),
+				     GTK_WINDOW (window));
 	gtk_window_set_modal (GTK_WINDOW (window), TRUE);
 
 	gtk_tree_view_set_reorderable (GTK_TREE_VIEW (treeview), FALSE);
@@ -1017,7 +1018,7 @@ prefs_homepage_current_button_clicked_cb (GtkWidget *button,
           (EPHY_EMBED_CONTAINER (window));
 	g_return_if_fail (embed != NULL);
 
-	set_homepage_entry (dialog, ephy_web_view_get_address (EPHY_GET_EPHY_WEB_VIEW_FROM_EMBED (embed)));
+	set_homepage_entry (dialog, ephy_web_view_get_address (ephy_embed_get_web_view (embed)));
 }
 
 static void
