@@ -21,6 +21,7 @@
 
 #include "config.h"
 
+#include "ephy-embed.h"
 #include "ephy-embed-container.h"
 #include "ephy-embed-shell.h"
 #include "ephy-embed-single.h"
@@ -93,7 +94,7 @@ window_cmd_file_print_preview (GtkAction *action,
           (EPHY_EMBED_CONTAINER (window));
 	g_return_if_fail (EPHY_IS_EMBED (embed));
 
-	ephy_web_view_set_print_preview_mode (EPHY_GET_EPHY_WEB_VIEW_FROM_EMBED (embed), TRUE);
+	ephy_web_view_set_print_preview_mode (ephy_embed_get_web_view (embed), TRUE);
 	_ephy_window_set_print_preview (window, TRUE);
 }
 
@@ -124,8 +125,8 @@ window_cmd_file_send_to	(GtkAction *action,
           (EPHY_EMBED_CONTAINER (window));
 	g_return_if_fail (embed != NULL);
 
-	location = ephy_web_view_get_address (EPHY_GET_EPHY_WEB_VIEW_FROM_EMBED (embed));
-	title = ephy_web_view_get_title (EPHY_GET_EPHY_WEB_VIEW_FROM_EMBED (embed));
+	location = ephy_web_view_get_address (ephy_embed_get_web_view (embed));
+	title = ephy_web_view_get_title (ephy_embed_get_web_view (embed));
 
 	subject = g_uri_escape_string (title, NULL, TRUE);
 	body = g_uri_escape_string (location, NULL, TRUE);
@@ -247,8 +248,8 @@ window_cmd_file_bookmark_page (GtkAction *action,
 	g_return_if_fail (embed != NULL);
 
 	ephy_bookmarks_ui_add_bookmark (GTK_WINDOW (window),
-					ephy_web_view_get_address (EPHY_GET_EPHY_WEB_VIEW_FROM_EMBED (embed)),
-					ephy_web_view_get_title (EPHY_GET_EPHY_WEB_VIEW_FROM_EMBED (embed)));
+					ephy_web_view_get_address (ephy_embed_get_web_view (embed)),
+					ephy_web_view_get_title (ephy_embed_get_web_view (embed)));
 }
 
 static void
@@ -327,6 +328,14 @@ window_cmd_file_save_as (GtkAction *action,
 	EphyEmbed *embed;
 	EphyFileChooser *dialog;
 
+	/* These all are needed in order to get the suggested filename */
+	char *suggested_filename;
+	const char *mimetype;
+	EphyWebView *view;
+	WebKitWebFrame *frame;
+	WebKitWebDataSource *data_source;
+	WebKitWebResource *web_resource;
+
 	embed = ephy_embed_container_get_active_child (EPHY_EMBED_CONTAINER (window));
 	g_return_if_fail (embed != NULL);
 
@@ -336,6 +345,29 @@ window_cmd_file_save_as (GtkAction *action,
 					CONF_STATE_SAVE_DIR,
 					EPHY_FILE_FILTER_ALL_SUPPORTED);
 
+	gtk_file_chooser_set_do_overwrite_confirmation (GTK_FILE_CHOOSER (dialog), TRUE);
+
+	view = ephy_embed_get_web_view (embed);
+	frame = webkit_web_view_get_main_frame (WEBKIT_WEB_VIEW (view));
+	data_source = webkit_web_frame_get_data_source (frame);
+	web_resource = webkit_web_data_source_get_main_resource (data_source);
+	mimetype = webkit_web_resource_get_mime_type (web_resource);
+
+	if ((g_ascii_strncasecmp (mimetype, "text/html", 9)) == 0)
+	{
+		/* Web Title will be used as suggested filename*/
+		suggested_filename = g_strconcat(ephy_web_view_get_title (view), ".html",NULL);
+	}
+	else
+	{
+		SoupURI *soup_uri = soup_uri_new (webkit_web_resource_get_uri (web_resource));
+		suggested_filename = g_path_get_basename (soup_uri->path);
+		soup_uri_free (soup_uri);
+	}
+
+	gtk_file_chooser_set_current_name (GTK_FILE_CHOOSER (dialog), suggested_filename);
+	g_free (suggested_filename);
+
 	g_signal_connect (dialog, "response",
 			  G_CALLBACK (save_response_cb), embed);
 
@@ -396,7 +428,7 @@ window_cmd_edit_undo (GtkAction *action,
 
 		if (embed)
 		{
-			webkit_web_view_undo (EPHY_GET_WEBKIT_WEB_VIEW_FROM_EMBED (embed));
+			webkit_web_view_undo (EPHY_GET_WEBKIT_WEB_VIEW_FROM_EMBED (EPHY_EMBED (embed)));
 		}
 	}
 }
@@ -421,7 +453,7 @@ window_cmd_edit_redo (GtkAction *action,
 		embed = gtk_widget_get_ancestor (widget, EPHY_TYPE_EMBED);
 		if (embed)
 		{
-			webkit_web_view_redo (EPHY_GET_WEBKIT_WEB_VIEW_FROM_EMBED (embed));
+			webkit_web_view_redo (EPHY_GET_WEBKIT_WEB_VIEW_FROM_EMBED (EPHY_EMBED (embed)));
 		}
 	}
 }
@@ -798,7 +830,7 @@ window_cmd_view_page_source (GtkAction *action,
           (EPHY_EMBED_CONTAINER (window));
 	g_return_if_fail (embed != NULL);
 
-	address = ephy_web_view_get_address (EPHY_GET_EPHY_WEB_VIEW_FROM_EMBED (embed));
+	address = ephy_web_view_get_address (ephy_embed_get_web_view (embed));
 	user_time = gtk_get_current_event_time ();
 
 	if (g_str_has_prefix (address, "file://"))
@@ -826,7 +858,7 @@ window_cmd_view_page_security_info (GtkAction *action,
           (EPHY_EMBED_CONTAINER (window));
 	g_return_if_fail (EPHY_IS_EMBED (embed));
 
-	ephy_web_view_show_page_certificate (EPHY_GET_EPHY_WEB_VIEW_FROM_EMBED (embed));
+	ephy_web_view_show_page_certificate (ephy_embed_get_web_view (embed));
 }
 
 void
@@ -854,7 +886,7 @@ window_cmd_edit_personal_data (GtkAction *action,
           (EPHY_EMBED_CONTAINER (window));
 	if (embed == NULL) return;
 
-	address = ephy_web_view_get_address (EPHY_GET_EPHY_WEB_VIEW_FROM_EMBED (embed));
+	address = ephy_web_view_get_address (ephy_embed_get_web_view (embed));
 	
 	host = address != NULL ? ephy_string_get_host_name (address) : NULL;
 
@@ -894,7 +926,7 @@ void
 window_cmd_help_contents (GtkAction *action,
 			 EphyWindow *window)
 {
-	ephy_gui_help (GTK_WINDOW (window), "epiphany", NULL);
+	ephy_gui_help (GTK_WIDGET (window), NULL);
 }
 
 #define ABOUT_GROUP "About"
